//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.IO;
using System.Drawing;
using System.Windows.Forms;


namespace MindFusion.Scheduling.WinForms.Samples.CS.Recurrence
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		/// <summary>
		/// Initializes a new instance of the MainForm class.
		/// </summary>
		public MainForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			_appPath = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			_lbInfo.DocumentText = Properties.Resources.Info;
			_lbInfo.DocumentCompleted += new WebBrowserDocumentCompletedEventHandler(OnDocumentCompleted);

			// Set the displayed date
			DateTime today = DateTime.Today;
			_calendar.Date = today;

			// Create the recurrence pattern (The third friday each month)
			_recurrence = new MindFusion.Scheduling.Recurrence();
			_recurrence.Pattern = RecurrencePattern.Monthly;
			_recurrence.MonthlyRecurrence = MonthlyRecurrence.ByDayType;
			_recurrence.Occurrence = Occurrence.Third;
			_recurrence.Day = DayOfWeekType.Friday;
			_recurrence.StartDate = new DateTime(today.Year, today.Month, 1);
			_recurrence.RecurrenceEnd = RecurrenceEnd.Never;

			// We have to associate the recurrence with an Item-derived
			// object to tell it what kind of objects to create in
			// GenerateItems method. For this purpose we use the
			// worker object below
			_worker = new RecurringDay();
			_worker.Recurrence = _recurrence;
		}

		private void OnDocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
		{
			if (_lbInfo.Document != null)
			{
				_lbInfo.Document.Click += Document_Click;
				_lbInfo.Document.Body.Style = "font-family: Verdana, Arial, Helvetica, sans-serif; " +
					"font-size: 7.5pt; margin: 2pt; padding: 0pt; background-color: #ffffee; cursor: default;";
			}
		}

		/// <summary>
		/// Handles the Web browser document's Click event.
		/// </summary>
		private void Document_Click(object sender, HtmlElementEventArgs e)
		{
			e.ReturnValue = true;

			HtmlElement pointed = _lbInfo.Document.GetElementFromPoint(e.ClientMousePosition);
			if (pointed == null)
				return;

			if (pointed.TagName.ToLower() == "a")
			{
				string href = pointed.GetAttribute("href").ToLower();
				if (href.StartsWith("help:"))
				{
					string topic = href.Substring(5);
					string helpFile = ResolveHelpPath(_appPath);

					if (System.IO.File.Exists(helpFile))
					{
						Help.ShowHelp(this, helpFile, HelpNavigator.Topic, topic);

						// Prevent link follow
						e.ReturnValue = false;
					}
				}
			}
		}

		private static string ResolveHelpPath(string baseDir)
		{
			string[] helpFiles = new string[]
			{
				"MindFusion.Scheduling.chm",
				"MindFusion WinForms.chm"
			};

			System.IO.DirectoryInfo dirInfo = new System.IO.DirectoryInfo(baseDir);
			int i = 0;
			while (dirInfo != null)
			{
				string helpFile = System.IO.Path.Combine(dirInfo.FullName, helpFiles[i]);
				if (System.IO.File.Exists(helpFile))
					return helpFile;

				i++;
				if (i == helpFiles.Length)
				{
					i = 0;
					dirInfo = dirInfo.Parent;
				}
			}

			return string.Empty;
		}

		private void OnAbout(object sender, System.EventArgs e)
		{
			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetAssembly(
				typeof(MindFusion.Scheduling.WinForms.Calendar));

			if (assembly == null)
				return;

			Type aboutFormType = assembly.GetType(
				"MindFusion.Scheduling.WinForms.AboutForm");

			if (aboutFormType == null)
				return;

			((Form)Activator.CreateInstance(aboutFormType)).ShowDialog(this);
		}

		private void OnExit(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void _schedule_Draw(object sender, MindFusion.Scheduling.WinForms.DrawEventArgs e)
		{
			if (e.Element == CustomDrawElements.CellHeader)
			{
				DateTime date = e.Date;

				// Check if this day falls within our recurrence pattern
				ItemCollection items = _recurrence.GenerateItems(date, date);

				bool isOccurrence = false;
				foreach (Item item in items)
				{
					if (item.StartTime.Date == date)
					{
						isOccurrence = true;
						break;
					}
				}

				if (isOccurrence)
				{
					// Do the custom drawing
					Rectangle bounds = e.Bounds;
					bounds.Width -= 1;
					bounds.Height -= 1;
					e.Graphics.DrawRectangle(Pens.Red, bounds);
				}
			}
		}


		private string _appPath;
		private MindFusion.Scheduling.Recurrence _recurrence;
		private RecurringDay _worker;
	}


	/// <summary>
	/// An item subclass used to generate occurrences upon.
	/// </summary>
	public class RecurringDay : Item
	{
		public RecurringDay() {}

		public override object Clone()
		{
			return null;
		}

		public override bool AllDayEvent
		{
			get { return false; }
			set { }
		}

		public override ContactCollection Contacts
		{
			get { return EmptyContacts; }
		}

		public override string DescriptionText
		{
			get { return ""; }
			set { }
		}

		public override string HeaderText
		{
			get { return ""; }
			set { }
		}

		public override DateTime StartTime
		{
			get { return _startTime; }
			set { _startTime = value; }
		}

		public override DateTime EndTime
		{
			get { return _endTime; }
			set { _endTime = value; }
		}

		public override string Id
		{
			get { return ""; }
			set { }
		}

		public override Location Location
		{
			get { return null; }
			set { }
		}

		public override Task Task
		{
			get { return null; }
			set { }
		}

		public override Reminder Reminder
		{
			get { return null; }
			set { }
		}

		public override bool Locked
		{
			get { return false; }
			set { }
		}

		public override Style Style
		{
			get { return null; }
			set { }
		}

		public override bool Visible
		{
			get { return false; }
			set { }
		}

		public override object Tag
		{
			get { return null; }
			set { }
		}

		public override Style SelectedStyle
		{
			get { return null; }
			set { }
		}

		public override ResourceCollection Resources
		{
			get { return EmptyResources; }
		}

		public override bool AllowChangeStart
		{
			get { return true; }
			set { }
		}

		public override bool AllowChangeEnd
		{
			get { return true; }
			set { }
		}

		public override bool AllowMove
		{
			get { return true; }
			set { }
		}

		public override Style PointedStyle
		{
			get { return null; }
			set { }
		}

		public override Style PointedSelectedStyle
		{
			get { return null; }
			set { }
		}


		private static ContactCollection EmptyContacts = new ContactCollection();
		private static ResourceCollection EmptyResources = new ResourceCollection();

		private DateTime _startTime;
		private DateTime _endTime;
	}
}
